<?php

namespace SM\Service;

use SM\Factory\ComposicaoServicosFactory;
use SM\Service\FindService;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use SM\Util\Procedure\FctSincronizarComposicao;
use Doctrine\ORM\EntityManager;

class ComposicaoServicosService
{
    protected $ComposicaoServicosFactory;
    protected $findService;
    protected $Environment;
    protected $newCode;
    protected $connection;

    public function __construct(EntityManager $entityManager, ComposicaoServicosFactory $ComposicaoServicosFactory, FindService $findService, Environment $Environment, NewCode $newCode)
    {
        $this->connection                   = $entityManager->getConnection();
        $this->ComposicaoServicosFactory    = $ComposicaoServicosFactory;
        $this->findService                  = $findService;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
    } 

    public function getComposicaoServicos()
    {
        return $this->ComposicaoServicosFactory->getComposicaoServicos();
    }

    public function getItComposicaoServico($parametros)
    {
        $nrorg = $this->Environment->getNrorg();

        $dataParams = array(
            "NRCOMPSERVICO" => $parametros["NRCOMPSERVICO"],
            "NRORG"         => $nrorg
        );

        return $this->ComposicaoServicosFactory->getItComposicaoServico($dataParams);
    } 

    public function getServicoOrcaTelaElemento()
    {
        return $this->ComposicaoServicosFactory->getServicoOrcaTelaElemento();
    } 

    public function getElementoServicoSuperior($parametros)
    {
        $nrorg = $this->Environment->getNrorg();

        $dataParams = array(
            "NRCOMPSERVICO"     => $parametros["NRCOMPSERVICO"],
            "NRITCOMPSERVICO"   => isset($parametros["NRITCOMPSERVICO"])? $parametros["NRITCOMPSERVICO"] : null,
            "NRORG"             => $nrorg
        );

        return $this->ComposicaoServicosFactory->getElementoServicoSuperior($dataParams);
    }

    public function saveComposicaoServicos($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrOrg();
            $cdoperador = $this->Environment->getCdOperador();
            $novoCodigo = $this->newCode->getCodeByTableName("FCT_COMPSERVICO", $this->Environment->getNrOrg(), 1, 10, true);

            //Nova Composição de Serviços
            if (!isset($params["NRCOMPSERVICO"])) {

                $dataParams = array(
                    "NRCOMPSERVICO"     => $novoCodigo,
                    "NMCOMPSERVICO"     => $params["NMCOMPSERVICO"],
                    "DSCOMPSERVICO"     => $params["DSCOMPSERVICO"],
                    "IDATIVO"           => $params["IDATIVO"],
                    "NRCOMPSERVICOPAI"  => $params["NRCOMPSERVICOPAI"],
                    "IDCOMPOBRIGATORIA" => $params["IDCOMPOBRIGATORIA"],
                    "CDCENTCUST"        => $params["CDCENTCUST"],
                    "NRORG"             => $nrorg,
                    "CDOPERINCLUSAO"    => $cdoperador,
                );

                $this->ComposicaoServicosFactory->insertComposicaoServicos($dataParams);
                $this->connection->commit();

            } else {
                // Edição de uma Composição de Serviços
                $dataParamsAux = array(
                    "NRCOMPSERVICO"     => $params["NRCOMPSERVICO"],
                    "IDATIVO"           => $params["IDATIVO"],
                    "CDCENTCUST"        => $params["CDCENTCUST"],
                    "NRORG"             => $nrorg,
                    "CDOPERADOR"        => $cdoperador,
                );
                
                $this->ComposicaoServicosFactory->replicarAlteracaoIdAtivo($dataParamsAux);

                $dataParams = array(
                    "NRCOMPSERVICO"     => $params["NRCOMPSERVICO"],
                    "NMCOMPSERVICO"     => $params["NMCOMPSERVICO"],
                    "DSCOMPSERVICO"     => $params["DSCOMPSERVICO"],
                    "IDATIVO"           => $params["IDATIVO"],
                    "NRCOMPSERVICOPAI"  => isset($params["NRCOMPSERVICOPAI"]) ? $params["NRCOMPSERVICOPAI"] : null,
                    "IDCOMPOBRIGATORIA" => $params["IDCOMPOBRIGATORIA"],
                    "CDCENTCUST"        => $params["CDCENTCUST"],
                    "NRORG"             => $nrorg,
                    "CDOPERULTATU"      => $cdoperador,
                );
                
                $this->ComposicaoServicosFactory->updateComposicaoServicos($dataParams);
                $this->connection->commit();
            }   
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteComposicaoEmCascata($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrOrg();

            foreach ($parametros as $parametroCompServico) {
                $dataParam = array( 
                    "NRCOMPSERVICO" => $parametroCompServico["NRCOMPSERVICO"],
                    "NRORG"         => $nrorg,
                );
                
                $this->ComposicaoServicosFactory->deleteComposicaoEmCascata($dataParam);
            }
            $this->connection->commit();

        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveElementoServico($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrOrg();
            $cdoperador = $this->Environment->getCdOperador();
            $novoCodigo = $this->newCode->getCodeByTableName("FCT_ITCOMPSERVICO", $this->Environment->getNrOrg(), 1, 10, true);

            if ($params["IDELEMENTO"] == $params["IDELEMENTOPAI"]) {
                throw new \Exception('Operação bloqueada! Um Elemento de Serviço não pode ser superior a ele mesmo.');
            }

            // Novo Elemento de Serviço
            if (!isset($params["NRITCOMPSERVICO"])) {

                $paramsValida = array(
                    "NRCOMPSERVICO"     => $params["NRCOMPSERVICO"],
                    "IDELEMENTO"        => isset($params["IDELEMENTO"])? $params["IDELEMENTO"] : null,
                    "NRORG"             => $nrorg,
                );
        
                $itemRepetido = $this->ComposicaoServicosFactory->verificarItemElementoRepetido($paramsValida);
        
                if ($itemRepetido[0]["EXISTE"] > 0) {
                    throw new \Exception("Operação Bloqueada! O elemento selecionado já existe nesta composição.");
                }

                // Valida se o elemento possui um pai, caso exista só poderá adiciona-lo com o respectivo pai.
                if (isset($params["IDELEMENTOPAI_VALIDACAO"])) {

                    $paramsValidaElementoPai = array(
                        "NRCOMPSERVICO"     => $params["NRCOMPSERVICO"],
                        "IDELEMENTO"        => $params["IDELEMENTOPAI_VALIDACAO"],
                        "NRORG"             => $nrorg,
                    );

                    $existeElementoPai = $this->ComposicaoServicosFactory->verificarItemElementoRepetido($paramsValidaElementoPai);

                    if (($existeElementoPai[0]["EXISTE"] > 0) && ($params["IDELEMENTOPAI_VALIDACAO"] == $params["IDELEMENTOPAI"])) {
                        
                        $dataParams = array(
                            "NRITCOMPSERVICO"   => $novoCodigo,
                            "NRCOMPSERVICO"     => $params["NRCOMPSERVICO"],
                            "CDSERVORCA"        => $params["CDSERVORCA"],
                            "IDSOMAVALORSUP"    => 'S',
                            "IDSERVOBRIGCOMP"   => $params["IDSERVOBRIGCOMP"],
                            "IDELEMENTO"        => $params["IDELEMENTO"],
                            "IDELEMENTOPAI"     => $params["IDELEMENTOPAI"],
                            "IDTIPOELEMENTO"    => $params["IDTIPOELEMENTO"],
                            "IDAPARECEPROPOSTA" => $params["IDAPARECEPROPOSTA"],
                            "IDATIVO"           => 'S',
                            "NRORG"             => $nrorg,
                            "CDOPERINCLUSAO"    => $cdoperador,
                        );

                        $this->ComposicaoServicosFactory->insertElementoServico($dataParams);
                        $this->connection->commit();
                        
                    } else {
                        throw new \Exception("Operação bloqueada. Um Elemento não pode ser inserido sem o seu Elemento Superior.");
                    }
                } else {

                    $dataParams = array(
                        "NRITCOMPSERVICO"   => $novoCodigo,
                        "NRCOMPSERVICO"     => $params["NRCOMPSERVICO"],
                        "CDSERVORCA"        => $params["CDSERVORCA"],
                        "IDSOMAVALORSUP"    => 'S',
                        "IDSERVOBRIGCOMP"   => $params["IDSERVOBRIGCOMP"],
                        "IDELEMENTO"        => $params["IDELEMENTO"],
                        "IDELEMENTOPAI"     => $params["IDELEMENTOPAI"],
                        "IDTIPOELEMENTO"    => $params["IDTIPOELEMENTO"],
                        "IDAPARECEPROPOSTA" => $params["IDAPARECEPROPOSTA"],
                        "IDATIVO"           => 'S',
                        "NRORG"             => $nrorg,
                        "CDOPERINCLUSAO"    => $cdoperador,
                    );
            
                    $this->ComposicaoServicosFactory->insertElementoServico($dataParams);
                    $this->connection->commit();
                }
                
            } else {

                // Edição de um elemento de serviço
                if ((isset($params["IDSERVOBRIGCOMP"])) && ($params["IDSERVOBRIGCOMP"] != $params["IDSERVOBRIGCOMP_BACKUP"] )) {
                    
                    $dataParamsReplicarObrigatoriedade = array(
                        "IDELEMENTO"      => $params["IDELEMENTO"],
                        "NRCOMPSERVICO"   => $params["NRCOMPSERVICO"],
                        "IDSERVOBRIGCOMP" => $params["IDSERVOBRIGCOMP"],
                        "NRORG"           => $nrorg,
                        "CDOPERULTATU"    => $cdoperador,
                        "NRORGULTATU"     => $nrorg
                    );
    
                    // Replica obrigatoriedade
                    $this->ComposicaoServicosFactory->replicarObrigatoriedadeItensFilhos($dataParamsReplicarObrigatoriedade);
                }

                $dataParamsUpdate = array(
                    "NRITCOMPSERVICO"   => $params["NRITCOMPSERVICO"],
                    "NRCOMPSERVICO"     => $params["NRCOMPSERVICO"],
                    "CDSERVORCA"        => $params["CDSERVORCA"],
                    "IDSOMAVALORSUP"    => 'S',
                    "IDSERVOBRIGCOMP"   => $params["IDSERVOBRIGCOMP"],
                    "IDELEMENTO"        => $params["IDELEMENTO"],
                    "IDELEMENTOPAI"     => $params["IDELEMENTOPAI"],
                    "IDTIPOELEMENTO"    => $params["IDTIPOELEMENTO"],
                    "IDAPARECEPROPOSTA" => $params["IDAPARECEPROPOSTA"],
                    "IDATIVO"           => 'S',
                    "NRORG"             => $nrorg,
                    "CDOPERULTATU"      => $cdoperador,
                );
                
                $this->ComposicaoServicosFactory->updateElementoServico($dataParamsUpdate);
                $this->connection->commit();
            }   
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteItemComposicaoEmCascata($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrOrg();

            foreach ($parametros as $parametroCompServico) {
                $dataParam = array( 
                    "NRCOMPSERVICO" => $parametroCompServico["NRCOMPSERVICO"],
                    "IDELEMENTO"    => $parametroCompServico["IDELEMENTO"],
                    "NRORG"         => $nrorg,
                );
                
                $this->ComposicaoServicosFactory->deleteItemComposicaoEmCascata($dataParam);
            }
            $this->connection->commit();

        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveCadastramentoAutomatico($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrOrg();
            $cdoperador = $this->Environment->getCdOperador();

            foreach ($params["SERVORCA_ARRAY"] as $servorca) {
                
                $paramsValida = array(
                    "NRCOMPSERVICO"     => $params["NRCOMPSERVICO"],
                    "IDELEMENTO"        => isset($servorca["IDELEMENTO"])? $servorca["IDELEMENTO"] : null,
                    "NRORG"             => $nrorg,
                );
        
                $itemRepetido = $this->ComposicaoServicosFactory->verificarItemElementoRepetido($paramsValida);
        
                if ($itemRepetido[0]["EXISTE"] > 0) {
                    throw new \Exception("Operação Bloqueada! O elemento selecionado já existe nesta composição.");
                }

                $novoCodigo = $this->newCode->getCodeByTableName("FCT_ITCOMPSERVICO", $this->Environment->getNrOrg(), 1, 10, true);

                $dataParams = array(
                    "NRITCOMPSERVICO"   => $novoCodigo,
                    "NRCOMPSERVICO"     => $params["NRCOMPSERVICO"],
                    "CDSERVORCA"        => $servorca["CDSERVORCA"],
                    "IDSOMAVALORSUP"    => 'S',
                    "IDSERVOBRIGCOMP"   => $params["IDSERVOBRIGCOMP"],
                    "IDELEMENTO"        => $servorca["IDELEMENTO"],
                    "IDELEMENTOPAI"     => $servorca["IDELEMENTOPAI"],
                    "IDTIPOELEMENTO"    => $servorca["IDTIPOELEMENTO"],
                    "IDAPARECEPROPOSTA" => $params["IDAPARECEPROPOSTA"],
                    "IDATIVO"           => 'S',
                    "NRORG"             => $nrorg,
                    "CDOPERINCLUSAO"    => $cdoperador,
                );
        
                $this->ComposicaoServicosFactory->insertElementoServico($dataParams);
            }

            $this->connection->commit();
            
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function sincronizarComposicao($params)
    {
        $nrcompservico = $params["NRCOMPSERVICO"];
        $nrorg         = $this->Environment->getNrOrg();

        $this->executarProcedureFctSincronizarComposicao($nrcompservico, $nrorg); 
    }

    /**
     * Executa a procedure FCT_SINCRONIZAR_COMPOSICAO.
     */
    public function executarProcedureFctSincronizarComposicao($nrcompservico, $nrorg) {
        $procedure = new FctSincronizarComposicao($this->connection);
        $procedure->executar($nrcompservico, $nrorg);
    }   
}